@@ -22,6 +22,14 @@ mboost_fit <- function(blg, response, weights = rep(1, NROW(response)),
     ngradient <- family@ngradient
     riskfct <- family@risk
 
+    ### weights not specified: unweighted problem
+    if (is.null(weights)) 
+        weights <- rep.int(1, NROW(response))
+    
+    ## oobweights not specified
+    if (is.null(oobweights))
+        oobweights <- as.numeric(weights == 0)
+    
     ### handle missing responses (via zero weights)
     yna <- is.na(response)
     y <- response
@@ -33,17 +41,13 @@ mboost_fit <- function(blg, response, weights = rep(1, NROW(response)),
                 " and thus these observations are not used for fitting")
     }
     y <- check_y_family(y, family)
-
-    ### unweighted problem
-    if (is.null(weights)) weights <- rep.int(1, NROW(y))
     if (!family@weights(weights))
         stop(sQuote("family"), " is not able to deal with weights")
 
     ### rescale weights (because of the AIC criterion)
     ### <FIXME> is this correct with zero weights??? </FIXME>
     weights <- rescale_weights(weights)
-    if (is.null(oobweights))
-        oobweights <- as.numeric(weights == 0)
+    
     if (control$risk == "oobag") {
         triskfct <- function(y, f) riskfct(y, f, oobweights)
     } else {
@@ -471,8 +475,10 @@ mboost_fit <- function(blg, response, weights = rep(1, NROW(response)),
 ### is evaluated as
 ###     y ~ bols3(x1) + baselearner(x2) + btree(x3)
 ### see mboost_fit for the dots
-mboost <- function(formula, data = list(), na.action = na.omit,
-    baselearner = c("bbs", "bols", "btree", "bss", "bns"), ...) {
+mboost <- function(formula, data = list(), na.action = na.omit, weights = NULL, 
+                   offset = NULL, family = Gaussian(), control = boost_control(),
+                   oobweights = NULL, baselearner = c("bbs", "bols", "btree", "bss", "bns"), 
+                   ...) {
 
 
     ## We need at least variable names to go ahead
@@ -485,10 +491,21 @@ mboost <- function(formula, data = list(), na.action = na.omit,
     }
 
     if (is.data.frame(data)) {
-        if (!all(Complete.cases(data))) {
+        if (!all(cc <- Complete.cases(data))) {
             ## drop cases with missing values in any of the specified variables:
             vars <- all.vars(formula)[all.vars(formula) %in% names(data)]
             data <- na.action(data[, vars])
+            
+            ## check if weights need to be removed as well
+            if (!is.null(weights) && nrow(data) < length(weights)) {
+                if (sum(cc) == nrow(data))
+                    weights <- weights[cc]
+            }
+            ## check if oobweights need to be removed as well
+            if (!is.null(oobweights) && nrow(data) < length(oobweights)) {
+                if (sum(cc) == nrow(data))
+                    oobweights <- oobweights[cc]
+            }
         }
     } else {
         if (any(unlist(lapply(data, function(x) !all(Complete.cases(x))))))
@@ -552,7 +569,11 @@ mboost <- function(formula, data = list(), na.action = na.omit,
     ### get the response
     response <- eval(as.expression(formula[[2]]), envir = data,
                      enclos = environment(formula))
-    ret <- mboost_fit(bl, response = response, ...)
+    
+    ret <- mboost_fit(bl, response = response, weights = weights, 
+                      offset = offset, family = family, 
+                      control = control, oobweights = oobweights, ...)
+    
     if (is.data.frame(data) && nrow(data) == length(response))
         ret$rownames <- rownames(data)
     else
@@ -562,8 +583,10 @@ mboost <- function(formula, data = list(), na.action = na.omit,
 }
 
 ### nothing to do there
-gamboost <- function(formula, data = list(),
-    baselearner = c("bbs", "bols", "btree", "bss", "bns"), dfbase = 4, ...) {
+gamboost <- function(formula, data = list(), na.action = na.omit, weights = NULL, 
+                     offset = NULL, family = Gaussian(), control = boost_control(),
+                     oobweights = NULL, baselearner = c("bbs", "bols", "btree", "bss", "bns"), 
+                     dfbase = 4, ...) {
 
     if (is.character(baselearner)) {
         baselearner <- match.arg(baselearner)
@@ -577,7 +600,11 @@ gamboost <- function(formula, data = list(),
     stopifnot(is.function(baselearner))
     if (isTRUE(all.equal(baselearner, bbs)))
         baselearner <- function(...) bbs(as.data.frame(list(...)), df = dfbase)
-    ret <- mboost(formula = formula, data = data, baselearner = baselearner, ...)
+    
+    ret <- mboost(formula = formula, data = data, na.action = na.action, 
+                  weights = weights, offset = offset, family = family, 
+                  control = control, oobweights = oobweights,
+                  baselearner = baselearner, ...)
     ret$call <- match.call()
     class(ret) <- c("gamboost", class(ret))
     ret
@@ -585,13 +612,17 @@ gamboost <- function(formula, data = list(),
 
 
 ### just one single tree-based baselearner
-blackboost <- function(formula, data = list(), weights = NULL,
-    na.action = na.pass,
-    tree_controls = party::ctree_control(teststat = "max",
-                               testtype = "Teststatistic",
-                               mincriterion = 0,
-                               maxdepth = 2, savesplitstats = FALSE),
-    ...) {
+blackboost <- function(formula, data = list(),
+                       weights = NULL, na.action = na.pass,
+                       offset = NULL, family = Gaussian(), 
+                       control = boost_control(),
+                       oobweights = NULL,
+                       tree_controls = party::ctree_control(
+                           teststat = "max",
+                           testtype = "Teststatistic",
+                           mincriterion = 0,
+                           maxdepth = 2, savesplitstats = FALSE),
+                       ...) {
 
     ### get the model frame first
     cl <- match.call()
@@ -608,7 +639,9 @@ blackboost <- function(formula, data = list(), weights = NULL,
     ## drop weights
     mf$"(weights)" <- NULL
     bl <- list(btree(mf, tree_controls = tree_controls))
-    ret <- mboost_fit(bl, response = response, weights = weights, ...)
+    ret <- mboost_fit(bl, response = response,  weights = weights, 
+                      offset = offset, family = family, 
+                      control = control, oobweights = oobweights, ...)
     ret$call <- cl
     ret$rownames <- rownames(mf)
     class(ret) <- c("blackboost", class(ret))
@@ -619,8 +652,10 @@ blackboost <- function(formula, data = list(), weights = NULL,
 glmboost <- function(x, ...) UseMethod("glmboost", x)
 
 glmboost.formula <- function(formula, data = list(), weights = NULL,
+                             offset = NULL, family = Gaussian(),
                              na.action = na.pass, contrasts.arg = NULL,
-                             center = TRUE, control = boost_control(), ...) {
+                             center = TRUE, control = boost_control(), 
+                             oobweights = NULL, ...) {
 
     ## We need at least variable names to go ahead
     if (length(formula[[3]]) == 1) {
@@ -671,8 +706,9 @@ glmboost.formula <- function(formula, data = list(), weights = NULL,
     bl <- list(bolscw(X))
     response <- model.response(mf)
     weights <- model.weights(mf)
-    ret <- mboost_fit(bl, response = response, weights = weights,
-                      control = control, ...)
+    ret <- mboost_fit(bl, response = response, weights = weights, 
+                      offset = offset, family = family, 
+                      control = control, oobweights = oobweights, ...)
     ret$newX <- newX
     ret$assign <- assign
     ret$center <- cm
@@ -728,8 +764,9 @@ glmboost.formula <- function(formula, data = list(), weights = NULL,
 }
 
 glmboost.matrix <- function(x, y, center = TRUE, weights = NULL,
-                            na.action = na.pass,
-                            control = boost_control(), ...) {
+                            offset = NULL, family = Gaussian(),
+                            na.action = na.pass, control = boost_control(), 
+                            oobweights = NULL, ...) {
 
     X <- x
     if (nrow(X) != NROW(y))
@@ -781,7 +818,9 @@ glmboost.matrix <- function(x, y, center = TRUE, weights = NULL,
         return(NULL)
     }
     bl <- list(bolscw(X))
-    ret <- mboost_fit(bl, response = y, control = control, ...)
+    ret <- mboost_fit(bl, response = y, weights = weights, 
+                      offset = offset, family = family, 
+                      control = control, oobweights = oobweights, ...)
     ret$newX <- newX
     ret$assign <- assign
     ret$center <- cm
